@@ -4,7 +4,7 @@ from dataclasses import (
 from redshift_client.data_type.core import (
     DataType,
 )
-from redshift_client.sql_client.core import (
+from redshift_client.sql_client import (
     PrimitiveVal,
 )
 
