@@ -32,7 +32,7 @@ def assert_fetch_one(
     raise TypeError(f"Unexpected fetch_one result; got {type(result)}")
 
 
-def assert_fetch_all(result: _T) -> FrozenList[FrozenList[PrimitiveVal]]:
+def assert_fetch_list(result: _T) -> FrozenList[FrozenList[PrimitiveVal]]:
     _assert: Callable[
         [_R], ResultE[FrozenList[PrimitiveVal]]
     ] = lambda l: to_list_of(l, to_prim_val)
