@@ -1,6 +1,9 @@
 from dataclasses import (
     dataclass,
 )
+from fa_purity import (
+    PureIter,
+)
 from fa_purity.cmd import (
     Cmd,
 )
@@ -39,8 +42,17 @@ from typing import (
     Callable,
     Dict,
     Optional,
+    TypeVar,
 )
 
+_T = TypeVar("_T")
+
+
+def _assert_bool(raw: _T) -> bool:
+    if isinstance(raw, bool):
+        return raw
+    raise TypeError("Expected bool")
+
 
 @dataclass(frozen=True)
 class ManifestId:
@@ -140,6 +152,76 @@ class TableClient:
 
         return (exe + results).map(_extract)
 
+    def exist(self, table_id: TableId) -> Cmd[bool]:
+        stm = """
+            SELECT EXISTS (
+                SELECT * FROM information_schema.tables
+                WHERE table_schema = %(table_schema)s
+                AND table_name = %(table_name)s
+            );
+        """
+        args: Dict[str, PrimitiveVal] = {
+            "table_schema": table_id.schema.name,
+            "table_name": table_id.name,
+        }
+        return self._db_client.execute(
+            new_query(stm), QueryValues(FrozenDict(args))
+        ) + self._db_client.fetch_one().map(
+            lambda m: m.map(lambda i: _assert_bool(i.data[0]))
+            .to_result()
+            .alt(lambda _: TypeError("Expected not None"))
+            .unwrap()
+        )
+
+    def insert(
+        self, table_id: TableId, table: Table, items: PureIter[RowData]
+    ) -> Cmd[None]:
+        _fields = ",".join(f"{{field_{i}}}" for i, _ in enumerate(table.order))
+        stm = f"""
+            INSERT INTO {{schema}}.{{table}} ({_fields}) VALUES %s
+        """
+        identifiers: Dict[str, Optional[str]] = {
+            "schema": table_id.schema.name,
+            "table": table_id.name,
+        }
+        for i, c in enumerate(table.order):
+            identifiers[f"field_{i}"] = c.name
+        return self._db_client.values(
+            dynamic_query(stm, FrozenDict(identifiers)), items
+        )
+
+    def rename(self, table_id: TableId, new_name: str) -> Cmd[TableId]:
+        stm = """
+            ALTER TABLE {schema}.{table} RENAME TO {new_name}
+        """
+        identifiers: Dict[str, Optional[str]] = {
+            "schema": table_id.schema.name,
+            "table": table_id.name,
+            "new_name": new_name,
+        }
+        return self._db_client.execute(
+            dynamic_query(stm, FrozenDict(identifiers)), None
+        ).map(lambda _: TableId(table_id.schema, new_name))
+
+    def _delete(self, table_id: TableId, cascade: bool) -> Cmd[None]:
+        _cascade = "CASCADE" if cascade else ""
+        stm = f"""
+            DROP TABLE {{schema}}.{{table}} {_cascade}
+        """
+        identifiers: Dict[str, Optional[str]] = {
+            "schema": table_id.schema.name,
+            "table": table_id.name,
+        }
+        return self._db_client.execute(
+            dynamic_query(stm, FrozenDict(identifiers)), None
+        )
+
+    def delete(self, table_id: TableId) -> Cmd[None]:
+        return self._delete(table_id, False)
+
+    def delete_cascade(self, table_id: TableId) -> Cmd[None]:
+        return self._delete(table_id, True)
+
     def new(
         self, table_id: TableId, table: Table, if_not_exist: bool = False
     ) -> Cmd[None]:
